
#include <MySensor.h>
#include <SPI.h>
#include <DHT.h>  

unsigned long SLEEP_TIME = 15000;
/*Child ID definition : 0:Door sensor   1: Humidity   2: Temperature */
#define CHILD_ID_HUM 1
#define CHILD_ID_TEMP 2
#define CHILD_ID 0

/* Pin definition : D3 Door( interrupt), 4 Humidity and temp */ 
#define BUTTON_PIN 3
#define HUMIDITY_SENSOR_DIGITAL_PIN 4

/* DHT setup */
DHT dht;
float lastTemp;
float lastHum;
boolean metric = true; 

/* DOOR SENSOR switch setup */
volatile boolean isOpen;
volatile boolean openFlag;
/* Message type Definition */
MyMessage msgHum(CHILD_ID_HUM, V_HUM);
MyMessage msgTemp(CHILD_ID_TEMP, V_TEMP);
MyMessage msg(CHILD_ID,V_TRIPPED);

MySensor gw;

void setup()  
{ 
  gw.begin();
  pinMode(BUTTON_PIN,INPUT);

  // Initialize library and add callback for incoming messages
  // Send the sketch version information to the gateway and Controller
  gw.sendSketchInfo("Door and Temp and Hum", "1.0");
  dht.setup(HUMIDITY_SENSOR_DIGITAL_PIN); 
  // Register all sensors to gw (they will be created as child devices)
  gw.present(CHILD_ID, S_DOOR);
  gw.present(CHILD_ID_HUM, S_HUM);
  gw.present(CHILD_ID_TEMP, S_TEMP);
  metric = gw.getConfig().isMetric;
  attachInterrupt(1,DoorMoving,CHANGE);

}

void DoorMoving(){
  isOpen=digitalRead(BUTTON_PIN)==LOW;   //1 if signal is LOW ( open door ) , 0 if signal is HIGH ( closed door)
  openFlag=true;
  return;
}

void loop() 
{  
  if (openFlag) {
     gw.send(msg.set(isOpen==HIGH ? 0 : 1));
     openFlag=false;
  }
  // Alway process incoming messages whenever possible
  delay(dht.getMinimumSamplingPeriod());
  // Alway process incoming messages whenever possible
  float temperature = dht.getTemperature();
  if (isnan(temperature)) {
    Serial.println("Failed reading temperature from DHT");
  } 
  else if (temperature != lastTemp) {
    lastTemp = temperature;
    if (!metric) {
      temperature = dht.toFahrenheit(temperature);
    }
    gw.send(msgTemp.set(temperature, 1));
    Serial.print("T: ");
    Serial.println(temperature);
  }

  float humidity = dht.getHumidity();
  if (isnan(humidity)) {
    Serial.println("Failed reading humidity from DHT");
  } 
  else if (humidity != lastHum) {
    if(humidity >= lastHum+2 || humidity <= lastHum-2){
    lastHum = humidity;
    gw.send(msgHum.set(humidity, 1));
    Serial.print("H: ");
    Serial.println(humidity);
    }
  }
 //gw.sleep(1,CHANGE,SLEEP_TIME);
 gw.sleep(SLEEP_TIME);
}


